/*
 * CONTRAT DE LICENCE DE LOGICIEL LIBRE CeCILL-B
 * https://cecill.info/licences/Licence_CeCILL-B_V1-fr.html
 */
package fr.unistra.sil.erp.back.db;

import fr.unistra.sil.erp.back.model.Category;
import fr.unistra.sil.erp.back.model.Item;
import fr.unistra.sil.erp.back.model.RegistryEntry;
import fr.unistra.sil.erp.back.model.Stock;
import java.util.List;

/**
 * Standard interface for database implementations.
 * 
 * This interface allows the application to be able to use different
 * database systems (CSV, SQLite, MySQL...) by simply switching
 * implementations.
 * @author BEAUVAIS ANTOINE
 */
public interface IDatabase {
    
    /**
     * Returns the list of all items.
     * @return the list of all items, or null if an error occurred.
     */
    public List<Item> getAllItems();
    
    /**
     * Returns the list of all items for a given category.
     * @param category the category's identifier.
     * @return the list of items, or null if an error occurred.
     */
    public List<Item> getItemsFromCategory(int category);
    
    /**
     * Returns the list of all categories.
     * 
     * @return the list of categories, or null if an error occurred.
     */
    public List<Category> getCategories();
    
    /**
     * Returns the list of all stocks.
     * 
     * @return the list of stocks.
     */
    public List<Stock> getStocks();
    
    /**
     * Returns the stock entry of an item.
     * @param itemId the item's ID.
     * @return the stock line.
     * @throws DatabaseResourceNotFoundException when the query fails.
     */
    public Stock getStockForItem(int itemId)
            throws DatabaseResourceNotFoundException;
    
    /**
     * Updates the specified stock's quantity.
     * @param id the stock entry's ID.
     * @param quantity the new quantity.
     * @throws DatabaseConnectionException when the database is unreachable.
     * @throws DatabaseUpdateException when the query fails.
     */
    public void updateStock(int id, int quantity)
            throws DatabaseConnectionException, DatabaseUpdateException;
    
    /**
     * Returns all registry entries.
     */
    public List<RegistryEntry> getRegistry() throws DatabaseConnectionException;
    
}
